/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "no22.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define NO22_DUMMY 0

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

void no22_generic_transfer ( no22_t *ctx, spi_master_transfer_data_t *block );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void no22_cfg_setup ( no22_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->pht = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

NO22_RETVAL no22_init ( no22_t *ctx, no22_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return NO22_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, NO22_DUMMY );

    // Output pins 

    digital_out_init( &ctx->pht, cfg->pht );

    return NO22_OK;

}

uint16_t no22_read_data ( no22_t *ctx )
{
    spi_master_transfer_data_t block;
    uint8_t rx_data[ 2 ];
    uint8_t temp_data = 0;

    spi_master_start( &ctx->spi );
    rx_data[ 0 ] = spi_master_read( &ctx->spi );
    rx_data[ 1 ] = spi_master_read( &ctx->spi );
    spi_master_stop( &ctx->spi );

    temp_data = rx_data[ 0 ];
    temp_data <<= 8;
    temp_data |= rx_data[ 1 ];

    temp_data &= 0x1FFF;
    temp_data >>= 1;


    return temp_data;
}

float no22_get_ppb ( no22_t *ctx )
{
    uint16_t adc_data;
    float temp;

    adc_data = no22_read_data( ctx );

    temp = adc_data / 4095.0;
    temp *= 1000;

    return temp;
}

uint8_t no22_set_pht_state ( no22_t *ctx, uint8_t state )
{
    if ( ( state == NO22_PIN_STATE_HIGH ) || ( state == NO22_PIN_STATE_LOW ) )
    {
        digital_out_write( &ctx->pht, state );
        
        return NO22_DEVICE_OK;
    }
    else
    {
        return NO22_DEVICE_ERROR;
    }
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

void no22_generic_transfer ( no22_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, block );
    spi_master_stop( &ctx->spi );    
}

// ------------------------------------------------------------------------- END

